@@ -1,5 +1,5 @@
 ## -*- truncate-lines: t; -*-
-## Copyright (C) 2008-18  Enrico Schumann
+## Copyright (C) 2008-20  Enrico Schumann
 
 pricetable <- function(price, ...)
     UseMethod("pricetable")
@@ -76,7 +76,7 @@ pricetable.zoo <- function(price, ..., instrument) {
 }
 
 `[.pricetable`  <- function(p, i, j, start, end, missing = NA,...,
-                            drop = FALSE) {
+                            drop = FALSE, as.matrix = TRUE) {
 
     ## pt[when, instrument]
     ##
@@ -137,9 +137,17 @@ pricetable.zoo <- function(price, ..., instrument) {
 
     if (!is.na(missing) && missing != "locf") {
         ans[is.na(ans)] <- missing
+    } else if (!is.na(missing) && (missing == "locf" || missing == "previous")) {
+        ans <- zoo::na.locf(ans, na.rm = FALSE)
+    }
+    if (!as.matrix)
+        class(ans) <- "pricetable"
+    else {
+        rownames(ans) <- as.character(attr(ans, "timestamp"))
+        colnames(ans) <- as.character(attr(ans, "instrument"))
+        attr(ans, "timestamp") <- NULL
+        attr(ans, "instrument") <- NULL
     }
-    class(ans) <- "pricetable"
-
     ans
 }
 
